local addon = WakeSpams
local module = addon:NewModule("Options")
WakeSpams.Options = module

local ACD = LibStub("AceConfigDialog-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")
local db

local options
local helptext = "#s# = spellname,\
#d# = duration,\
#t# = targetname,\
#amount# = amount (heal, damage, etc),\
#maxhp# = your maximum HP,\
#halfhp# = half of your maximum HP,\
#extra# = resist type for resists, spell names for interrupted spells, etc.\
#extra:SpellLink# = same as above, but with spell links if available."

function module:RegisterOptions()
	-- xD
	options = {
		name = 'WakeSpams',
		type = "group",
		args = {
			general = {
				name = 'WakeSpams',
				type = "group",
				args = {
					enabled = {
						order = 1,
						type = "toggle",
						name = "Enabled",
						desc = "Enable or disable addon.",
						descStyle = "inline",
						set = function(v) db.profile.enabled = not db.profile.enabled end,
						get = function() return db.profile.enabled end,
					},
					spellheader = {
						name = "Spell Announcements",
						type = "header",
						order = 10,
					},
					aoe = {
						order = 11,
						type = "toggle",
						name = "AoE",
						desc = "Enable or disable announcements for multitarget spells.",
						descStyle = "inline",
						set = function(v) db.profile.aoe = not db.profile.aoe end,
						get = function() return db.profile.aoe end,
					},
					interrupts = {
						order = 12,
						type = "toggle",
						name = "Interrupts",
						desc = "Enable or disable announcements for interrupts.",
						descStyle = "inline",
						set = function(i,v)
							if not db.profile.EventList["SPELL_INTERRUPT"] then
								db.profile.EventList["SPELL_INTERRUPT"] = {}
							end
							db.profile.EventList["SPELL_INTERRUPT"].disabled = not v
						end,
						get = function()
							if not db.profile.EventList["SPELL_INTERRUPT"] then
								return true
							else
								return not db.profile.EventList["SPELL_INTERRUPT"].disabled
							end
						end,
					},
					dispels = {
						order = 13,
						type = "toggle",
						name = "Dispels",
						desc = "Enable or disable announcements for successful dispels.",
						descStyle = "inline",
						set = function(i,v)
							if not db.profile.EventList["SPELL_DISPEL"] then
								db.profile.EventList["SPELL_DISPEL"] = {}
							end
							db.profile.EventList["SPELL_DISPEL"].disabled = not v
						end,
						get = function()
							if not db.profile.EventList["SPELL_DISPEL"] then
								return true
							else
								return not db.profile.EventList["SPELL_DISPEL"].disabled
							end
						end,
					},
					dispelresists = {
						order = 14,
						type = "toggle",
						name = "Dispel Resists",
						desc = "Enable or disable announcements for dispel resists.",
						descStyle = "inline",
						set = function(i,v)
							if not db.profile.EventList["SPELL_DISPEL_FAILED"] then
								db.profile.EventList["SPELL_DISPEL_FAILED"] = {}
							end
							db.profile.EventList["SPELL_DISPEL_FAILED"].disabled = not v
						end,
						get = function()
							if not db.profile.EventList["SPELL_DISPEL_FAILED"] then
								return true
							else
								return not db.profile.EventList["SPELL_DISPEL_FAILED"].disabled
							end
						end,
					},
					defaults = {
						name = "Edit Defaults",
						type = "header",
						order = 20,
					},
					AbilityStart = {
						name = "",
						type = "group",
						inline = true,
						order = 21,
						args = {
							input = {
								order = 3,
								type = "input",
								name = "Default ability start text",
								desc = "#s# #d# etc xDD",
								width = "full",
								set = function(i,v) db.profile.AbilityStart = v end,
								get = function() return db.profile.AbilityStart end,
							},
						},
					},
					AbilityEnd = {
						name = "",
						type = "group",
						inline = true,
						order = 22,
						args = {
							input = {
								order = 3,
								type = "input",
								name = "Default ability end text",
								desc = "#s# #d# etc xDD",
								width = "full",
								set = function(i,v) db.profile.AbilityEnd = v end,
								get = function() return db.profile.AbilityEnd end,
							},
						},
					},
					strreplace = {
						name = "String replacements",
						type = "header",
						order = 30,
					},
					strreplacedesc = {
						name = helptext,
						type = "description",
						fontSize = "medium",
						order = 31,
					},
				},
			},
		},
	}
	
	local function EditSpell(spellid)
		local sname, _, sicon, _, _, _, _, _, _ = GetSpellInfo(spellid)
		local spell = addon.SpellList[spellid]
		if not db.profile.SpellList[spellid] then
			db.profile.SpellList[spellid] = {}
		end
		local tbl = {
			name = "Edit Spell",
			type = "group",
			order = 1,
			args = {
				description = {
					name = spell.configname or sname,
					fontSize = "large",
					type = "description",
					order = 1,
					width = "full",
					image = sicon,
					imageCoords = {0.1,0.9,0.1,0.9},
					imageWidth = 20,
					imageHeight = 20,
				},
				desctwo = {
					name = "|cffffae00Spell ID: "..spellid.."|r",
					fontSize = "",
					type = "description",
					order = 2,
					width = "full",
				},
				spacerone = {
					name = "Spell Behaviour",
					type = "header",
					order = 3,
				},
				duration = {
					name = "Duration of the spell (in seconds)",
					desc = "Enter the duration of the spell here. The addon doesn't look up durations automatically, so if you have a talent that increases the duration, you will have to change it manually.",
					type = "range",
					min = 0,
					max = 1800,
					softMax = 40,
					step = 1,
					order = 30,
					set = function(info,val) db.profile.SpellList[spellid].dur = val end,
					get = function() return db.profile.SpellList[spellid].dur or spell.dur end,
				},
				aoe = {
					name = "AoE spell",
					desc = "Toggle this if it's a spell that hits multiple targets. Toggling this will also ignore custom start and end messages.",
					type = "toggle",
					order = 31,
					disabled = function() return (db.profile.SpellList[spellid].custom) and nil or true end,
					set = function(info,val) db.profile.SpellList[spellid].aoe = val end,
					get = function() return db.profile.SpellList[spellid].aoe or spell.aoe end,
				},
				countdown = {
					name = "Countdown in chat",
					desc = "Counts down in the chat when the spell is about to fade. Set to 0 for no countdown.",
					type = "range",
					min = 0,
					max = 5,
					step = 1,
					order = 32,
					set = function(info,val) db.profile.SpellList[spellid].countdown = val end,
					get = function() return db.profile.SpellList[spellid].countdown or spell.countdown end,
				},
				spellfade = {
					name = "Announce when the spell fades",
					desc = "Toggle to announce when the spell fades.",
					type = "toggle",
					order = 33,
					set = function(info,val) db.profile.SpellList[spellid].countdown = (val) and 0 or nil end,
					get = function() return db.profile.SpellList[spellid].countdown or spell.countdown end,
				},
				whisper = {
					name = "Whisper target",
					desc = "Toggle this if you wish to whisper the target of the spell.",
					type = "toggle",
					order = 40,
					set = function(info,val) db.profile.SpellList[spellid].whisper = val end,
					get = function() return db.profile.SpellList[spellid].whisper or spell.whisper end,
				},
				spacertwo = {
					name = "Chat Customization",
					type = "header",
					order = 49,
				},
				txt = {
					name = "Start message of spell. Leave blank for default.",
					type = "input",
					order = 50,
					width = "full",
					disabled = function() return (spell.aoe) and true or nil end,
					set = function(info,val) db.profile.SpellList[spellid].txt = val end,
					get = function() return db.profile.SpellList[spellid].txt or spell.txt end,
				},
				txttwo = {
					name = "End message of spell. Leave blank for default.",
					type = "input",
					order = 51,
					width = "full",
					disabled = function() return (spell.aoe) and true or nil end,
					set = function(info,val) db.profile.SpellList[spellid].txtend = val end,
					get = function() return db.profile.SpellList[spellid].txtend or spell.txtend end,
				},
				output = {
					name = "Output",
					type = "select",
					order = 52,
					values = {
						DEFAULT = "Default",
					},
					style = "dropdown",
					set = function(info,val) db.profile.SpellList[spellid].output = val end,
					get = function() return db.profile.SpellList[spellid].output or "DEFAULT" end,
				},
				buttonOkay = {
					name = "Okay",
					type = "execute",
					order = 100,
					width = "half",
					func = function()
						options.args.spells.args.edit = nil
					end,
				},
				buttonDefaults = {
					name = "Defaults",
					type = "execute",
					order = 101,
					width = "half",
					func = function() db.profile.SpellList[spellid] = {} end,
				},
			},
		}
		return tbl
	end
	
	local function AddSpellDetails(spellid)
		local tbl
		local spell
		spellid = tonumber(spellid)
		if not addon:isNumeric(spellid) then -- Not a number
			tbl = {
				name = "Error: The entered Spell ID is not a number.",
				type = "description",
				order = 2,
				fontSize = "large",
			}
		elseif not GetSpellInfo(spellid) then -- Spell does not exist
			tbl = {
				name = ("Error: Could not find spell with the following Spell ID: %s"):format(spellid),
				type = "description",
				order = 2,
				fontSize = "large",
			}
		elseif ((addon.SpellList[spellid]) or (db.profile.SpellList[spellid])) then -- Already in the list
			spell = GetSpellInfo(spellid)
			tbl = {
				name = "",
				type = "group",
				inline = true,
				order = 2,
				args = {
					description = {
						name = ("Error: \"%s\" is already in the spell list. Do you wish to edit that spell instead?"):format(spell),
						type = "description",
						order = 1,
						fontSize = "large",
					},
					editbtn = {
						name = "Edit",
						type = "execute",
						order = 2,
						func = function()
							if(options.args.spells.args.oldlist) then
								options.args.spells.args.list = options.args.spells.args.oldlist
								options.args.spells.args.oldlist = nil
							else
								options.args.spells.args.oldlist = options.args.spells.args.list
								options.args.spells.args.list = nil
							end
							options.args.spells.args.edit = EditSpell(spellid)
						end,
					},
				},
			}
		else
			tbl = {
				name = "",
				type = "group",
				inline = true,
				order = 2,
				args = {
					description = {
						name = "Ok",
						type = "description",
						order = 1,
						fontSize = "large",
					},
				},
			}
		end
		return tbl
	end
	
	local function AddSpell()
		local spellid
		local tbl = {}
		tbl = {
			description = {
				name = "Add Spell",
				type = "group",
				inline = true,
				order = 1,
				args = {
					spellid = {
						name = "Spell ID. You can find Spell IDs for specific spells on WoWhead.",
						type = "input",
						order = 1,
						width = "full",
						get = function() return spellid or "Enter Spell ID #" end,
						set = function(i,v) spellid = v tbl.spellDetails = AddSpellDetails(spellid) return v end,
					},
				},
			},
			spellDetails = {
				name = "Enter a spell ID",
				type = "description",
				order = 2,
				fontSize = "large",
			},
		}
		return tbl
	end
	
	local function PopulateClassOptions()
		local tbl
		tbl = {
			name = 'spells',
			type = "group",
			desc = "Spells",
			childGroups = "tab",
			args = {
				add = {
					name = "Add Spell",
					type = "group",
					desc = "Add spell",
					order = 3,
					args = AddSpell(),
				},
				list = {
					name = "Spell List",
					type = "group",
					desc = "Spell list",
					order = 2,
					args = {
					},
				},
			},
		}
		if not addon.SpellList then
			local class, _ = UnitClass("player");
			tbl.args.list.args.info = {
				type = "description",
				name = ("There are no preconfigured %s spells. To add a spell, click the  \"Add Spell\" button."):format(class),
				order = 100,
			}
			return tbl
		end
		for i,v in pairs(addon.SpellList) do
			local sname, _, sicon, _, _, _, _, _, _ = GetSpellInfo(i)
			local spell = addon.SpellList[i]
			tbl.args.list.args[""..i..""] = {
				name = spell.configname or sname,
				type = "group",
				inline = true,
				order = i,
				args = {
					description = {
						name = "",
						type = "description",
						order = 1,
						image = sicon,
						width = "half",
						imageCoords = {0.1,0.9,0.1,0.9},
						imageWidth = 18,
						imageHeight = 18,
					},
					toggleSpell = {
						name = "Enable",
						type = "toggle",
						order = 2,
						width = "half",
						desc = GetSpellLink(i)..' Enable/disable announcing for this spell.',
						--descStyle = "inline",
						set = function(info,val)
							if not db.profile.SpellList[i] then
								db.profile.SpellList[i] = {}
							end
							db.profile.SpellList[i].disabled = not val
						end,
						get = function()
							if not db.profile.SpellList[i] then
								return true
							else
								return not db.profile.SpellList[i].disabled
							end
						end,
					},
					editSpell = {
						name = "Edit",
						type = "execute",
						order = 3,
						width = "half",
						desc = GetSpellLink(i)..' Edit this spell.',
						func = function()
							if(options.args.spells.args.oldlist) then
								options.args.spells.args.list = options.args.spells.args.oldlist
								options.args.spells.args.oldlist = nil
							else
								options.args.spells.args.oldlist = options.args.spells.args.list
								options.args.spells.args.list = nil
							end
							options.args.spells.args.edit = EditSpell(i)
						end,
					}
				}
			}
		end
		return tbl
	end
	options.args.spells = PopulateClassOptions()
	
	options.args.help = {
		type = "group",
		name = "Help",
		args = {
			strreplace = {
				name = "String replacements",
				type = "header",
				order = 1,
			},
			strreplacedesc = {
				name = helptext,
				type = "description",
				fontSize = "medium",
				order = 2,
			},
		},
	}
	
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)
end

function module:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.InterfaceOptions)
end

function module:OnInitialize()
	db = addon.db
	self:RegisterOptions()
	ACR:RegisterOptionsTable("WakeSpams", options)
	
	self.InterfaceOptions = ACD:AddToBlizOptions("WakeSpams", "WakeSpams", nil, "general")
	local class, _ = UnitClass("player");
	ACD:AddToBlizOptions("WakeSpams", class, "WakeSpams", "spells")
	ACD:AddToBlizOptions("WakeSpams", "Help", "WakeSpams", "help")
	ACD:AddToBlizOptions("WakeSpams", "Profiles", "WakeSpams", "profiles")
	LibStub("LibAboutPanel").new("WakeSpams", "WakeSpams")
end